@@ -96,7 +96,7 @@ use vars qw( $VERSION );
 use constant TRUE  => 1;
 use constant FALSE => 0;
 
-$VERSION = '3.001_02';
+$VERSION = '3.001_03';
 
 use DBI;
 
@@ -224,8 +224,17 @@ sub _decrypt {
 		$sha256 = substr $plaintext, 0, 32, '';
 		}
 
+	# padding is always added to get to a multiple of 16. If the value is already
+	# a multiple of 16, it's padded by an additional 16 octets.
 	my $padding_count = ord( substr $plaintext, -1 );
-	substr( $plaintext, -$padding_count ) = '' if $padding_count < 16;
+
+	my $padding = substr( $plaintext, -$padding_count );
+
+	unless( $padding =~ /\A(.){$padding_count}\z/ and ord(substr $padding, 0, 1) == $padding_count ) {
+		warnings::warn("Unexpected padding in encrypted cookie") if warnings::enabled();
+		}
+
+	substr( $plaintext, -$padding_count ) = '' if $padding_count <= 16;
 
 	( $plaintext, $sha256 );
 	}
@@ -245,7 +254,10 @@ sub _encrypt {
 
 	my $blocksize = 16;
 
+	# padding is always added. If the length is already a multiple of 16, the
+	# padding is the same as the blocksize.
 	my $padding_length = ($blocksize - length($value) % $blocksize);
+	$padding_length = $blocksize if $padding_length == 0;
 	my $padding = chr($padding_length) x $padding_length;
 	my $encrypted = 'v10' . $self->_cipher->encrypt( $value . $padding );
 
@@ -273,7 +285,6 @@ sub _filter_cookies {
 			$self->_insert( @parts );
 			}
 		);
-
 	}
 
 sub _get_rows {
@@ -658,16 +669,15 @@ specify a port. This version of C<set_cookie> does no port check.
 # We have to override this part because Chrome has -1 as a valid
 # port value (for "unspecified port"). Otherwise this is lifted from
 # HTTP::Cookies
-sub set_cookie
-{
+sub set_cookie {
 	my $self = shift;
 	my($version,
-	   $key, $val, $path, $domain, $port,
-	   $path_spec, $secure, $maxage, $discard, $rest) = @_;
+		$key, $val, $path, $domain, $port,
+		$path_spec, $secure, $maxage, $discard, $rest) = @_;
 
 	# path and key can not be empty (key can't start with '$')
 	return $self if !defined($path) || $path !~ m,^/, ||
-	            !defined($key)  || $key  =~ m,^\$,;
+		!defined($key)  || $key  =~ m,^\$,;
 
 	# ensure legal port
 	if (0 && defined $port) {  # nerf this part
@@ -676,11 +686,11 @@ sub set_cookie
 
 	my $expires;
 	if (defined $maxage) {
-	if ($maxage <= 0) {
-	    delete $self->{COOKIES}{$domain}{$path}{$key};
-	    return $self;
-	}
-	$expires = time() + $maxage;
+		if ($maxage <= 0) {
+			delete $self->{COOKIES}{$domain}{$path}{$key};
+			return $self;
+		}
+		$expires = time() + $maxage;
 	}
 	$version = 0 unless defined $version;
 
